/*-
 * Copyright (c) 2011 aian
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <cassert>
#include <algorithm>

#include <libqie/gray_image.h>

namespace qie {

gray_image::gray_image() : width_(0), height_(0) {
}

gray_image::gray_image(const gray_image& other) {
  alloc(other.width_, other.height_);
  std::copy(other.pixels_.begin(), other.pixels_.end(), pixels_.begin());
}

gray_image::~gray_image() {
}

gray_image&
gray_image::operator=(const gray_image& rhs) {
  if (this != &rhs) {
    gray_image temp(rhs);
    temp.swap(*this);
  }
  return *this;
}

unsigned char
gray_image::get_pixel(int x, int y) const {
  assert(is_alloced());
  assert(is_valid_coord(x, y));
  return pixels_.at(y * width_ + x); 
}

void
gray_image::set_pixel(int x, int y, unsigned char value) {
  assert(is_alloced());
  assert(is_valid_coord(x, y));
  pixels_.at(y * width_ + x) = value;
}

void
gray_image::fill(unsigned char value) {
  assert(is_alloced());
  std::fill(pixels_.begin(), pixels_.end(), value);
}

void
gray_image::alloc(int width, int height, unsigned char value) {
  assert(width > 0);
  assert(height > 0);
  if (is_alloced()) {
    clear();
  }
  std::size_t buffer_size = width * height;
  pixels_.resize(buffer_size);
  width_ = width;
  height_ = height;
  fill(value);
}

void
gray_image::clear() {
  pixels_.clear();
  width_ = 0;
  height_ = 0;
}

void
gray_image::swap(gray_image& other) throw () {
  std::swap(width_, other.width_);
  std::swap(height_, other.height_);
  pixels_.swap(other.pixels_);
}

int
gray_image::width() const throw () {
  return width_;
}

int
gray_image::height() const throw () {
  return height_;
}

bool
gray_image::is_alloced() const throw () {
  return pixels_.size() > 0;
}

bool
gray_image::is_valid_coord(int x, int y) const throw () {
  return (x >= 0) && (y >= 0) && (x < width_) && (y < height_);
}

}
