/*-
 * Copyright (c) 2011 aian
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <cassert>
#include <cstdlib>
#include <fstream>
#include <stdexcept>

#include <libqie/gray_image.h>

#include <libqie/pnm_header.h>
#include <libqie/pgm_reader.h>

namespace qie {

pgm_reader::pgm_reader() {
}

pgm_reader::~pgm_reader() {
}

void
pgm_reader::read(const std::string& fname, gray_image& image) {
  std::ifstream ifs(fname.c_str(), std::ios::binary | std::ios::in);
  if (ifs.fail()) {
    throw std::runtime_error("file open error.");
  }
  pnm_header header;
  parse_header(ifs, header);
  prepare_image(header, image);
  switch (header.format()) {
  case pnm_format_ascii_graymap:
    parse_ascii_pixels(ifs, image);
    break;
  case pnm_format_binary_graymap:
    parse_binary_pixels(ifs, image);
    break;
  default:
    assert(false);
    break;
  }
}

void
pgm_reader::parse_header(std::ifstream& ifs, pnm_header& header) const {
  parse_format_type(ifs, header);
  parse_image_size(ifs, header);
  parse_pixel_depth(ifs, header);
}

void
pgm_reader::parse_format_type(std::ifstream& ifs, pnm_header& header) const {
  std::string token;
  get_token(ifs, token);
  header.set_format(token);
}

void
pgm_reader::parse_image_size(std::ifstream& ifs, pnm_header& header) const {
  std::string token;
  get_token(ifs, token);
  header.set_width(std::strtol(token.c_str(), 0, 10));
  get_token(ifs, token);
  header.set_height(std::strtol(token.c_str(), 0, 10));
}

void
pgm_reader::parse_pixel_depth(std::ifstream& ifs, pnm_header& header) const {
  std::string token;
  get_token(ifs, token);
  header.set_depth(std::strtol(token.c_str(), 0, 10));
}

void
pgm_reader::parse_ascii_pixels(std::ifstream& ifs, gray_image& image) const {
  std::string token;
  unsigned char pixel = 0;
  for (int y = 0; y < image.height(); ++y) {
    for (int x = 0; x < image.width(); ++x) {
      get_token(ifs, token);
      pixel = static_cast<unsigned char>(std::strtol(token.c_str(), 0, 10));
      image.set_pixel(x, y, pixel);
    }
  }
}

void
pgm_reader::parse_binary_pixels(std::ifstream& ifs, gray_image& image) const {
  char pixel = 0;
  for (int y = 0; y < image.height(); ++y) {
    for (int x = 0; x < image.width(); ++x) {
      ifs.get(pixel);
      image.set_pixel(x, y, static_cast<unsigned char>(pixel));
    }
  }
}

void
pgm_reader::prepare_image(const pnm_header& header, gray_image& image) const {
  // TODO: validate header member
  image.alloc(header.width(), header.height());
}

void
pgm_reader::get_token(std::ifstream& ifs, std::string& token) const {
  bool is_inside_token = false;
  std::string temp;
  char ch = 0;
  while (ifs.get(ch).good()) {
    if (is_inside_token) {
      if (std::isspace(ch) || ch == '#') {
        break;
      } else {
        temp.push_back(ch);
      }
    } else {
      if (std::isspace(ch)) {
        // do nothing ... continue.
      } else if (ch == '#') {
        skip_until_newline(ifs);
      } else {
        is_inside_token = true;
        ifs.unget();
      }
    }
  }
  token.swap(temp);
}

void
pgm_reader::skip_until_newline(std::ifstream& ifs) const {
  char ch = 0;
  while (ifs.get(ch).good()) {
    if (ch == '\n') {
      break;
    }
  }
}

}
