/*-
 * Copyright (c) 2011 aian
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <cassert>
#include <fstream>
#include <stdexcept>

#include <libqie/gray_image.h>
#include <libqie/pgm_writer.h>

namespace qie {

pgm_writer::pgm_writer() {
  // default header
  header_.set_format(pnm_format_binary_graymap);
  header_.set_width(0);
  header_.set_height(0);
  header_.set_depth(0xFF);
}

pgm_writer::~pgm_writer() {
}

void
pgm_writer::set_header(const pnm_header& header) {
  if (!is_valid_header(header)) {
    throw std::logic_error("invalid header.");
  }
  header_ = header;
}

void
pgm_writer::write(const std::string& fname, const gray_image& image) const {
  assert(image.is_alloced());
  std::ofstream ofs(fname.c_str(), std::ios::binary | std::ios::out);
  if (ofs.fail()) {
    throw std::runtime_error("file open error.");
  }
  // header
  write_header(ofs, image);
  // pixels
  switch (header_.format()) {
  case pnm_format_ascii_graymap:
    write_ascii_pixels(ofs, image);
    break;
  case pnm_format_binary_graymap:
    write_binary_pixels(ofs, image);
    break;
  default:
    assert(false);
    throw std::logic_error("invalid format type.");
    break;
  }
}

void
pgm_writer::write_header(std::ofstream& ofs, const gray_image& image) const {
  ofs << header_.format_string() << std::endl;
  ofs << image.width() << " " << image.height() << std::endl;
  ofs << header_.depth() << std::endl;
}

void
pgm_writer::write_ascii_pixels(std::ofstream& ofs, const gray_image& image) const {
  for (int y = 0; y < image.height(); ++y) {
    for (int x = 0; x < image.width(); ++x) {
      ofs << static_cast<int>(image.get_pixel(x, y)) << std::endl;
    }
  }
}

void
pgm_writer::write_binary_pixels(std::ofstream& ofs, const gray_image& image) const {
  for (int y = 0; y < image.height(); ++y) {
    for (int x = 0; x < image.width(); ++x) {
      ofs.put(static_cast<char>(image.get_pixel(x, y)));
    }
  }
}

bool
pgm_writer::is_valid_header(const pnm_header& header) const throw () {
  bool is_valid = true;
  // format check
  if (is_valid) {
    switch (header.format()) {
    case pnm_format_ascii_graymap:
    case pnm_format_binary_graymap:
      break;
    default:
      is_valid = false;
    }
  }
  // depth check
  if (is_valid) {
    if (header.depth() != 0xFF) {
      is_valid = false;
    }
  }
  return is_valid;
}

}
